
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <xmltok/xmlparse.h>

typedef enum
{
	NONE,
	PERSON,
	BIRTH,
	NAME
} elemType;

elemType Current;

/* ----------
 * Callback for element begin
 * ----------
 */
void 
startElement(void *userData, const char *name, const char **atts)
{
	const char **x = atts;

	if (!(strcmp(name, "test:person")))		/* person */
	{
		printf("PERSON:\n");
		Current = PERSON;
	}
	else if (!(strcmp(name, "test:birth")))		/* birth */
	{
		printf("BIRTH:\n");
		Current = BIRTH;
	}
	else if (!(strcmp(name, "test:name")))		/* name */
	{
		printf("NAME:\n");
		Current = NAME;
	}
	else {
		Current = NONE;
		return;
	}
	
	while(x && *x)
	{
		char *key   = (char *)x[0];
		char *value = (char *)x[1];
	
		printf("\t%s: %s\n", key, value);
		x += 2;
	}
}  


/* ----------
 * Callback for element end
 * ----------
 */
void 
endElement(void *userData, const char *name)
{
	Current = NONE;
}

/* ----------
 * Callback for data inside element
 * ----------
 */
void 
CharacterDataHandler(void *userData, const XML_Char *s, int len)
{	
	char	buf[BUFSIZ];

	if (Current != NAME)
		return;

	if (!s || !*s)
		return;
	
	memset(buf, 0, BUFSIZ);
	strncpy(buf, s, len);
	
	printf("%s\n", buf);
}

int 
main(int argc, char **argv)
{
	FILE		*myfile;
	char		buf[BUFSIZ];
	XML_Parser 	parser = XML_ParserCreate(NULL);
	int 		done;

	if (argc < 1)
	{
		fprintf(stderr, "Usage: %s <file.xml>\n", argv[0]);
		exit(1);
	}	
	
	XML_SetCharacterDataHandler(parser, CharacterDataHandler);
	XML_SetElementHandler(parser, startElement, endElement);

	myfile = fopen( argv[1], "r");

	do {
		size_t len = fread(buf, 1, sizeof(buf), myfile);
		done = len < sizeof(buf);
	
		if (!XML_Parse(parser, buf, len, done)) {
      			fprintf(stderr, "%s at line %d\n",
			      XML_ErrorString(XML_GetErrorCode(parser)),
			      XML_GetCurrentLineNumber(parser));
			return 1;
		}
	} while (!done);
	
	XML_ParserFree(parser);
	fclose(myfile);
	return 0;
}
