
#include <stdio.h>
#include <string.h>

#include <gnome-xml/parser.h>

extern int xmlDoValidityCheckingDefaultValue;

/* ----------
 * Parse name
 * ----------
 */
void
parseName(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) 
{
	char	*type,
		*name;
	
	type = xmlGetProp(cur, "type");
	name = xmlNodeListGetString(doc, cur->childs, 1);
	
	printf("NAME:\n\ttype: %s\n\t%s\n", type, name);
}

/* ----------
 * Parse birth
 * ----------
 */
void
parseBirth(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) 
{
	char	*year,
		*month,
		*day;
	
	year = xmlGetProp(cur, "year");
	month= xmlGetProp(cur, "month");
	day  = xmlGetProp(cur, "day");
	
	printf("BIRTH:\n\tyear: %s\n\tmonth: %s\n\tday: %s\n", 
						year, month, day);
}

/* ----------
 * Parese person
 * ----------
 */
void
parsePerson(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) 
{
	printf("PERSON:\n\tname: %s\n", xmlGetProp(cur, "name"));

	cur = cur->childs;

	while (cur != NULL) 
	{
        	if ((!strcmp(cur->name, "birth")) && (cur->ns == ns)) 
	    		parseBirth(doc, ns, cur);
	    		
		else if ((!strcmp(cur->name, "name")) && (cur->ns == ns))
			parseName(doc, ns, cur);

        	cur = cur->next;		 
	}
}

/* ----------
 * Parse file
 * ----------
 */
int
parseFile(char *filename) 
{
	xmlDocPtr	doc;
	xmlNsPtr	ns;
	xmlNodePtr	cur;

	/*
	 * build an XML tree from a the file;
	 */
	doc = xmlParseFile(filename);
	if (doc == NULL) 	
		return 0;
	
	/*
	 * Check the document is of the right kind
	 */
	cur = doc->root;
	if (cur == NULL) 
	{
		fprintf(stderr,"Empty document\n");
		xmlFreeDoc(doc);
		return 0;
    	}
    
    	/*
    	 * Check namespice
    	 */
    	ns = xmlSearchNsByHref(doc, cur, "http://www.nikde.cz/");
    	if (ns == NULL) 
    	{
        	fprintf(stderr,
		        "document of the wrong type, Test Namespace not found\n");
		xmlFreeDoc(doc);
		return 0;
	}
    
    	if (strcmp(cur->name, "lw"))
    	{
    		fprintf(stderr, "Root tag must be 'lw'!\n");
    		return 0;
    	}
    	
	cur = cur->childs;

	while (cur != NULL) 
	{
    		if ((!strcmp(cur->name, "person")) && (cur->ns == ns))
			parsePerson(doc, ns, cur);
    		cur = cur->next;
    	}
    	
    	return 1;
}

int 
main(int argc, char **argv) 
{    
	int	i;

	/* xmlDoValidityCheckingDefaultValue = 1; */

	if (argc < 1)
	{
		fprintf(stderr, "Usage: %s <file.xml>\n", argv[0]);
		exit(1);
	}

	for (i = 1; i < argc ; i++) 
	{
		if (!parseFile(argv[i]))
			fprintf(stderr, "Can't parse file '%s'\n", argv[i]); 
	}
	return(0);
}
